﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using PaymentEngine;

namespace Payment_Engine_Demo_App
{
    public partial class Main : Form
    {
        private Double total;
        private TransactionRequest tran;
        private Middleware client;
        private Device device;

        private String ApiKey;
        private String ApiPin;
        private String DeviceKey;

        public Main()
        {
            InitializeComponent();


            total = 0;
            tran = new TransactionRequest();

            lblDeviceModel.Text = "No device connected";
            lblDeviceStatus.Text = "";
            btnTakePayment.Enabled = false;


            // read save values from registry.  DONT DO THIS.  
            // In a production app you should store these values encrypted.
            Microsoft.Win32.RegistryKey key;
            key = Microsoft.Win32.Registry.CurrentUser.CreateSubKey("PayEngineDemo");
            try { ApiKey = (String) key.GetValue("ApiKey"); }
            catch (Exception) { ApiKey = ""; }
            try { ApiPin = (String)key.GetValue("ApiPin"); }
            catch (Exception) { ApiPin = ""; }
            try { DeviceKey = (String)key.GetValue("DeviceKey"); }
            catch (Exception) { DeviceKey = ""; }

            //ApiKey = "0hxEwTCy4UPSTp7sL5V69fa2Uu5Gzrfu";
            //ApiPin = "1234";
            //DeviceKey = "sa_54kx8PXr0YDdkgFvaftqEveVT3JME";


        }

        private void button1_Click(object sender, EventArgs e)
        {

        }

        private void toolStripStatusLabel2_Click(object sender, EventArgs e)
        {

        }

        private void btnAddItem1_Click(object sender, EventArgs e)
        {
            addItem("Coffee", 1.55);
        }

        private void btnAddItem2_Click(object sender, EventArgs e)
        {
            addItem("Eggs", 2.95);
        }

        private void btnAddItem3_Click(object sender, EventArgs e)
        {
            addItem("Toast", 1.25);
        }

        private void btnAddItem4_Click(object sender, EventArgs e)
        {
            addItem("Oatmeal", 2.25);
        }

        private void addItem(String Name, Double Amount)
        {
            total += Amount;
            listItems.Items.Add("$" + Amount + " " + Name);
            lblTotal.Text = "$" + total;

            LineItem line = new LineItem()
            {
                Name = Name,
                Cost = Amount.ToString(),
                Quantity = "1",
                SKU = "99999"
            };



            //this.request.LineItems[] = line;

            if (total>0)
            {
                btnTakePayment.Enabled = true;
            }

        }

        private void btnClearOrder_Click(object sender, EventArgs e)
        {
            total = 0;
            lblTotal.Text = "$0.00";
            listItems.Items.Clear();
        }

        private void btnSettings_Click(object sender, EventArgs e)
        {
            Settings settings = new Settings((key, pin, devicekey) =>
            {
                ApiKey = key;
                ApiPin = pin;
                DeviceKey = devicekey;

                Console.WriteLine("Saving ApiKey " + ApiKey);

                using (Microsoft.Win32.RegistryKey reg = Microsoft.Win32.Registry.CurrentUser.CreateSubKey("PayEngineDemo"))
                {
                    try
                    {
                        reg.SetValue("ApiKey", ApiKey);
                        reg.SetValue("ApiPin", ApiPin);
                        reg.SetValue("DeviceKey", DeviceKey);

                    }
                    catch (Exception ex) {
                        Console.WriteLine("Failed to save to registry: " + ex.Message);
                    }
                }

            });

            settings.txtSourceKey.Text = ApiKey;
            settings.txtPin.Text = ApiPin;
            settings.txtDeviceKey.Text = DeviceKey;

            settings.ShowDialog();

        }

        private void ToggleConnectButton(bool connect)
        {
            if(connect)
            {
                setDeviceModel("Device not connected");
                setDeviceStatus("");
                btnConnect.Text = "Connect";
                btnConnect.Enabled = true;
            } else
            {
                btnConnect.Text = "Disconnect";
                btnConnect.Enabled = true;

            }

        }

        private void btnConnect_Click(object sender, EventArgs e)
        {

            if(btnConnect.Text.Equals("Disconnect"))
            {
                if(device != null)
                {
                    device.OnStatusChange(null);
                    device = null;
                }
                client = null;

                lblDeviceModel.Text = "Device not connected";
                lblDeviceStatus.Text = "";
                btnConnect.Text = "Connect";
                btnConnect.Enabled = true;
                return;
            }

            try
            {
                if (ApiKey == null) throw new Exception("Api Key not set");
                if (ApiKey.Length==0) throw new Exception("Api Key not set");
                if (ApiPin == null) throw new Exception("Api Pin not set");
                if (ApiPin.Length == 0) throw new Exception("Api Pin not set");
            }
            catch (Exception ex)
            {
                lblDeviceStatus.Text = ex.Message;
                return;
            }

            try
            {
                client = new Middleware(ApiKey, ApiPin, "www.usaepay.com");
                btnConnect.Text = "Disconnect";
            }
            catch (MiddlewareException ex)
            {
                lblDeviceStatus.Text = "Unable to connect to gateway";
                Console.WriteLine("Error instantiating middleware: " + ex.ToString());
            }

            // start pairing process
            if(DeviceKey.Trim().Length == 0)
            {
                lblDeviceStatus.Text = "Pairing device...";
                PairDevice pair = new PairDevice(client);
                pair.Show(this);
                Console.WriteLine("showDialog() done");
                pair.startRegistration((dev) =>
                {
                    DeviceKey = dev.DeviceKey;
                    device = dev;
                    OnDeviceRefresh();
                });
                return;
            }

            // load existing device key
            try
            {
                lblDeviceStatus.Text = "Connecting to device...";
                client.LoadDeviceByKey(DeviceKey, (dev) =>
                {

                    // store device
                    device = dev;

                    // update device status ui
                    statusStrip1.Items[2].Text = "Serial # " + dev.Details.Serial;
                    statusStrip1.Items[0].Text = dev.Status;
                    OnDeviceRefresh(device.Status);

                    // register for future status changes
                    device.OnStatusChange(OnDeviceRefresh);

                }, (emw) =>
                {
                    setDeviceStatus(emw.Message);
                    Console.WriteLine("Error loading device: " + emw.Message + " [" + emw.ErrorCode + "]");
                });
            }
            catch (MiddlewareException ex)
            {
                Console.WriteLine("Error loading device: " + ex.Message);
            }


        }

        public void PairingCancled()
        {
            ToggleConnectButton(true);
        }

        private void OnDeviceRefresh(String stat=null)
        {
            Console.WriteLine("OnDeviceRefresh triggered: " + stat);
       
            if(device == null)
            {
                setDeviceModel("Device not connected");
                setDeviceStatus(stat);
                return;
            }

            setDeviceModel(device.Details.Model);
            setDeviceStatus(device.Status);
        }

        private void setDeviceStatus(String status)
        {
            if (lblDeviceStatus.InvokeRequired)
            {
                lblDeviceStatus.Invoke(new MethodInvoker(delegate { lblDeviceStatus.Text = status; }));
            }
            else
            {
                lblDeviceStatus.Text = status;
            }
        }


        private void setDeviceModel(String status)
        {
            if (lblDeviceModel.InvokeRequired)
            {
                lblDeviceModel.Invoke(new MethodInvoker(delegate { lblDeviceModel.Text = status; }));
            }
            else
            {
                lblDeviceModel.Text = status;
            }
        }


        private void startTransaction(Boolean manualKey=false, String command="cc:sale")
        {

            if(device == null || client == null)
            {
                setDeviceStatus("Take device online before starting payment");
                return;
            }

            tran.Command = command;
            tran.Amount = total.ToString();
            tran.Invoice = "10001";
            tran.ManualKey = manualKey;

            Process proc = new Process(device);
            proc.Show();
            Console.WriteLine("proc show() returned");
            proc.startTransaction(tran);



        }


        /*
         * Start a payment request.  This will send the transaction data to
         * the terminal and prompt the customer to swipe/dip/tap their card.
         * Once the payment is complete,  transaction results will be returned.
         * See: Process.cs
         */
        private void btnTakePayment_Click(object sender, EventArgs e)
        {
            startTransaction(false);
        }


        /*
         * Trigger the payment request with a flag to prompt the customer
         * to enter their card number instead of swipe/dip
         */
        private void btnManuallyKey_Click(object sender, EventArgs e)
        {
            startTransaction(true);
        }


        /*
         * This demonstrates an unconnected refund.  There is no reference to the original sale.
         * An amount is provided and the customer is prompted to swipe/dip  their card.
         */
        private void btnRefund_Click(object sender, EventArgs e)
        {
            startTransaction(false, "cc:credit");

        }

    }

}
